/*
  This program keeps the aspect ratio and the size of the drawn
  image at a constant 500 by 500 pixels. Also, the drawn image is
  anchored in the upper left hand corner of this program's window.
  But the program window can be any size. If the program window is
  smaller than 500 by 500 pixels, then only part of the drawn image
  is shown in the program window (the view of our image in world
  coordinats gets "cropped" to fit inside of the screen window).

  Run this program and resize the window so that it is smaller than
  500 by 500 and see what happens.

  To create this effect, we need to use the reshape() callback function
  and in the diplay() function we need to use both the glViewport() and
  gluOrtho2D() functions. That is, we need to control the world window
  in world coordinates (using the gluOrtho2D() function) and the viewport
  within the screen window (using the glViewport() function). As the user
  changes the shape of the screen window, we need to change the shape of
  the world-window to always be the same as the screen window.
*/
#include <stdio.h>
#include <math.h>

#if defined _WIN32 || defined _WIN64
#  include <windows.h>
#endif

#ifdef __APPLE__
#  include <GLUT/glut.h>
#else
#  include <GL/glut.h>
#endif

/* Prototypes for the display() and reshape() callback functions */
void display(void);
void reshape(int, int);


const int screenSize = 500;
int windowHeight;              // this is set by the reshape() callback function
int windowWidth;               // this is set by the reshape() callback function

int main(int argc, char** argv)
{
   /* Standard GLUT initialization */
   glutInit(&argc,argv);
   glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB); /* default, not needed */
   glutInitWindowSize(500, 500);     /* 500 x 500 pixel window */
   glutInitWindowPosition(100, 100); /* place window near top left of display */
   glutCreateWindow("Circle v3");    /* window title */
   glutDisplayFunc(display);         /* display callback invoked when window is redrawn */
   glutReshapeFunc(reshape);         /* reshape callback invoked when window is resized */

   /* Initialize OpenGL state */
   glShadeModel(GL_FLAT);            /* default is GL_SMOOTH */
   glFrontFace(GL_CCW);
   glPolygonMode(GL_FRONT, GL_LINE);
   glPolygonMode(GL_BACK, GL_FILL);
   glClearColor(1.0, 1.0, 1.0, 1.0); /* white background */
   glColor3f(1.0, 0.0, 0.0);         /* draw in red */

   /* Enter event loop */
   glutMainLoop();

   return 0;  /* we never get here */
}//main()


/* The display() callback function */
void display( void )
{
   const double PI = 3.14159265;
   int numPoints = 100;
   int n;
   int viewportLLx, viewportLLy, viewportWidth, viewportHeight;
   double left, right, bottom, top;

   /* set up viewing (world window) */
   /*
      The "world window" is at most -2<=x<=2 and -2<=y<=2.
      If the program's window is less than 500 by 500 pixels,
      then the aspect ratio of the world window is the same
      as the program's window (and the viewport) and the world
      window's upper left hand corner is always the point with
      world coordinates (-2,2).
   */
   left   = -2.0;
   right  = ( windowWidth < screenSize) ? -2.0 + (4.0* windowWidth/screenSize) :  2.0;
   bottom = (windowHeight < screenSize) ?  2.0 - (4.0*windowHeight/screenSize) : -2.0;
   top    = 2.0;
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluOrtho2D(left, right, bottom, top); /* viewing volume */
   glMatrixMode(GL_MODELVIEW);

   /* specify the viewport */
   /*
      The viewport is never more than 500 by 500 pixels and it is always
      anchored in the upper left hand corner of the program's screen window.
      If the program's screen window is less than 500 by 500 pixels, then
      the viewport is the same size as the screen window.
   */
   viewportWidth  = (windowWidth  <= screenSize) ? windowWidth  : screenSize;
   viewportHeight = (windowHeight <= screenSize) ? windowHeight : screenSize;
   viewportLLx = 0;
   viewportLLy = (windowHeight <= screenSize) ? 0 : windowHeight - screenSize;
   glViewport(viewportLLx, viewportLLy, viewportWidth, viewportHeight);

   glClear(GL_COLOR_BUFFER_BIT);  /* clear the screen buffer */

   glBegin(GL_TRIANGLE_FAN);
     glVertex2f(0.0, 0.0);
     for (n = 0; n <= numPoints; n++)
        glVertex2f(cos((n/(double)numPoints)*2*PI), sin((n/(double)numPoints)*2*PI));
   glEnd();

   glFlush();

   printf("We got a \"display()\" event\n");
}//display()


/* The reshape() callback function */
void reshape(int width, int height)
{
   windowWidth  = width;
   windowHeight = height;

   printf("We got a \"reshape(%d, %d)\" event\n", width, height);

   display();  // Try commenting out this line. What effect does ithat have?
}//reshape()
