/*
   This version adjusts the viewport and the world window so that the aspect ratio of the
   graph is preserved (and the shape of the graph is not distorted) and also so that the
   entire height of the screen window is always used. This means that when the screen
   window is too narrow, part of the left and right ends of the graph must be cut off.
   Notice that "cutting off" the left and right ends of the graph is done by adjusting the
   world window.
*/
#include <stdio.h>
#include <math.h>

#if defined _WIN32 || defined _WIN64
#  include <windows.h>
#endif

#ifdef __APPLE__
#  include <GLUT/glut.h>
#else
#  include <GL/glut.h>
#endif

#define graphTop      1.0
#define graphBottom  -0.22
#define graphLeft    -4.0
#define graphRight   ( - graphLeft )
#define graphWidth   ( graphRight - graphLeft )
#define graphHeight  ( graphTop - graphBottom )
#define aspectRatio  ( graphWidth/graphHeight )

void reshape(int, int);

int windowHeight;    // this is set by the reshape() callback function
int windowWidth;     // this is set by the reshape() callback function

void display(void)   // plot the sinc function, using world coordinates
{
   const float pi = 3.14159265358979;
   float x;
   int viewportLeft, viewportBottom, viewportWidth, viewportHeight;
   float worldLeftEdge, worldRightEdge;

   glClear( GL_COLOR_BUFFER_BIT );

   // specify the viewport
   if ( (float)windowWidth/(float)windowHeight >= aspectRatio )
   {
      viewportWidth = (int)(windowHeight*aspectRatio);
      viewportLeft = (windowWidth-viewportWidth)/2;
   }
   else
   {
      viewportWidth = windowWidth;
      viewportLeft  = 0;
   }
   viewportHeight = windowHeight;  // always use the entire height of the screen window
   viewportBottom = 0;
   glViewport(viewportLeft, viewportBottom, viewportWidth, viewportHeight);

   // specify the world window
   if ( (float)windowWidth/(float)windowHeight >= aspectRatio )
   {
      worldLeftEdge = graphLeft;
   }
   else
   {
      if (viewportHeight != 0)  // check for minimized screen window
      {
         worldLeftEdge =  -(((float)viewportWidth/(float)viewportHeight)*graphHeight)/2.0;
      }
      else
      {
         worldLeftEdge = 0;
      }
   }
   printf("worldLeftEdge = %f, viewportHeight = %d\n", worldLeftEdge, viewportHeight);

   worldRightEdge = - worldLeftEdge;

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluOrtho2D(worldLeftEdge, worldRightEdge, graphBottom, graphTop);

   glBegin(GL_LINE_STRIP);
      for(x = worldLeftEdge; x < worldRightEdge; x += 0.01)  // draw the graph
      {
         glVertex2f( x, sin(pi*x)/(pi*x) );
      }
   glEnd();

   glFlush();
}


int main(int argc, char** argv)
{
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB);
   // specify the screen window
   glutInitWindowSize(640, 480);       // width, height (in pixels)
   glutInitWindowPosition(100, 150);   // left, top (in pixels)
   glutCreateWindow("Sinc Function v3");
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);

   glClearColor(1.0,1.0,1.0,0.0);
   glColor3f(0.0f,0.0f,1.0f);
   glLineWidth(1.0);

   glutMainLoop( );

   return 0;
}


/* The reshape() callback function */
void reshape(int width, int height)
{
   windowWidth  = width;
   windowHeight = height;

   display();
   printf("We got a \"reshape(%d, %d)\" event\n", width, height);
}//reshape()